/* eslint-disable no-unused-vars */
/* eslint-disable no-console */
/* eslint-disable no-undef */
import Render from './index.rjs';
import { getFileSystemManager } from '@ray-js/api';

Component({
  properties: {
    title: {
      type: String,
      value: 'hello RJS',
    },
    /**
     * 钢笔颜色
     */
    strokeColor: {
      type: String,
      value: '#000',
    },
    /**
     * 钢笔宽度
     */
    lineWidth: {
      type: Number,
      value: 2,
    },
    /**
     * 字体类型
     */
    font: {
      type: String,
      value: '60px Arial',
    },
    /**
     * 字体样式
     */
    fillStyle: {
      type: String,
      value: 'black',
    },
    /**
     * 图片地址
     */
    src: {
      type: String,
      value: '',
    },
    /**
     * 缩放rgba阵列数组成多少倍 > 1 为缩小
     */
    dest: {
      type: Number,
      value: 1,
    },
    /**
     * 输入的文字
     */
    text: {
      type: String,
      value: '',
    },
  },
  observers: {
    src(newVal, oldVal) {
      this.rjs.cancelAnimationFrame();
      this.clearCanvas();
      newVal && this.handleSrcToBase64(newVal);
    },
    text(newVal, oldVal) {
      this.rjs.cancelAnimationFrame();
      this.clearCanvas();
      newVal && this.handleWritreText(newVal);
    },
  },
  data() {
    return {
      isDrawing: false,
      lastX: 0,
      lastY: 0,
      canvasStyles: {},
      locations: [],
      base64: '',
    };
  },
  // 生命周期
  lifetimes: {
    ready() {
      this.rjs = new Render(this);
      this.rjs.initCanvas(this.data.dest);
      this.rjs.getCanvasStyles();
    },
  },
  methods: {
    // 获取canvas的样式
    getCanvasStyles(params) {
      this.setData({
        canvasStyles: params,
      });
    },
    // 处理手势按下事件
    startDrawing(e) {
      // this.clearCanvas();
      const { x, y } = e.changedTouches[0];
      this.setData({
        isDrawing: true,
        lastX: x,
        lastY: y,
        // locations: [{ x: Math.floor(x), y: Math.floor(y) }],
      });
    },
    // 处理手势移动事件
    async handleMouseMove(e) {
      const { isDrawing, lastX, lastY, canvasStyles, locations, strokeColor, lineWidth } =
        this.data;
      const { x, y } = e.changedTouches[0];

      if (!isDrawing) return;
      await this.rjs.freehandDraw(lastX, lastY, x, y, strokeColor, lineWidth);
      this.setData({
        lastX: x,
        lastY: y,
      });

      if (x <= canvasStyles.width && x >= 0 && y >= 0 && y <= canvasStyles.height) {
        this.setData({
          // locations: [{ x: Math.floor(x), y: Math.floor(y) }, ...locations],
        });
      }
    },
    // 处理手势抬起事件
    async endDrawing(e) {
      await this.setData({
        isDrawing: false,
      });
    },
    // 处理清空画布内容
    clearCanvas() {
      this.rjs.clearCanvas();
    },
    // 将图片src转化为base64
    handleSrcToBase64(newVal) {
      // this.rjs.pictureDraw(newVal, this.data.canvasStyles.width, this.data.canvasStyles.height);
      getFileSystemManager().readFile({
        filePath: this.data.src,
        encoding: 'base64',
        success: data => {
          this.setData(
            {
              base64: 'data:image/png;base64,' + data.data,
            },
            () => {
              this.rjs.pictureDraw(
                newVal,
                this.data.canvasStyles.width,
                this.data.canvasStyles.height
              );
            }
          );
        },
      });
    },
    // 处理获取字体大小
    handleMapFontSize() {
      const size = this.data.font.split(' ')[0];
      // 使用正则表达式匹配并替换掉所有非数字字符
      const result = size.replace(/[^0-9]/g, '');
      return result * 1;
    },
    // 处理输入文字
    handleWritreText(
      text,
      width = 0,
      height = this.data.canvasStyles.height / 2 + this.handleMapFontSize() / 4
    ) {
      this.rjs.writeTextDraw(text, width, height, this.data.font, this.data.fillStyle);
    },
  },
});
